package space.blokk.chat

import com.squareup.moshi.FromJson
import com.squareup.moshi.ToJson

@Suppress("ClassName")
sealed class ChatColor(val stringRepresentation: String) {
    object BLACK : ChatColor("black")
    object DARK_BLUE : ChatColor("dark_blue")
    object DARK_GREEN : ChatColor("dark_green")
    object DARK_AQUA : ChatColor("dark_aqua")
    object DARK_RED : ChatColor("dark_red")
    object PURPLE : ChatColor("dark_purple")
    object GOLD : ChatColor("gold")
    object GRAY : ChatColor("gray")
    object DARK_GRAY : ChatColor("dark_gray")
    object BLUE : ChatColor("blue")
    object GREEN : ChatColor("green")
    object AQUA : ChatColor("aqua")
    object RED : ChatColor("red")
    object PINK : ChatColor("light_purple")
    object YELLOW : ChatColor("yellow")
    object WHITE : ChatColor("white")

    class Hex(code: String) : ChatColor("#" + code.removePrefix("#")) {
        val code = toString()

        init {
            if (!HEX_COLOR_REGEX.matches(code)) throw IllegalArgumentException("code is not a valid hexadecimal color")
        }
    }

    override fun toString(): String = stringRepresentation

    companion object {
        private val HEX_COLOR_REGEX = Regex("^#(?:[0-9a-fA-F]{3}){1,2}\$")

        val NAMED_COLORS by lazy {
            setOf(
                BLACK,
                DARK_BLUE,
                DARK_GREEN,
                DARK_AQUA,
                DARK_RED,
                PURPLE,
                GOLD,
                GRAY,
                DARK_GRAY,
                BLUE,
                GREEN,
                AQUA,
                RED,
                PINK,
                YELLOW,
                WHITE
            ).map { it.stringRepresentation to it }.toMap()
        }

        fun fromString(value: String): ChatColor =
            if (value.startsWith("#")) Hex(value)
            else try {
                NAMED_COLORS.getValue(value)
            } catch (e: NoSuchElementException) {
                throw IllegalArgumentException("There is no color named '${value}'")
            }
    }

    object JSONAdapter {
        @ToJson
        fun toJson(value: ChatColor) = value.toString()

        @FromJson
        fun fromJson(value: String) = fromString(value)
    }
}

val String.color get() = ChatColor.fromString(this)
