package space.blokk.entity

import space.blokk.world.block.Block
import space.blokk.world.block.Material
import kotlin.reflect.KClass

interface EntityType<T : Entity> {
    val entityClass: KClass<T>
    val numericID: Int
    val id: String
    val width: Float
    val height: Float

    companion object {
        /**
         * All entity types, sorted by their numeric ID in ascending order.
         */
        val all = GENERATED_ENTITIES
        val byID = GENERATED_ENTITIES.map { it.id to it }.toMap()

        private val byClass = GENERATED_ENTITIES.map { it.entityClass to it }.toMap()

        @Suppress("UNCHECKED_CAST")
        fun <T: Entity> byClass(entityClass: KClass<T>): EntityType<T>? = byClass[entityClass] as EntityType<T>?
        inline fun <reified T: Entity> byClass(): EntityType<T>? = byClass(T::class)
    }
}
