package space.blokk.server

import space.blokk.Registry
import space.blokk.Scheduler
import space.blokk.command.Command
import space.blokk.event.EventBus
import space.blokk.event.EventHandlerPositionManager
import space.blokk.logging.Logger
import space.blokk.logging.LoggingOutputProvider
import space.blokk.net.Session
import space.blokk.player.Player
import space.blokk.plugin.PluginManager
import space.blokk.recipe.Recipe
import space.blokk.world.BiomeRegistry
import space.blokk.world.Dimension
import java.io.File
import kotlin.coroutines.CoroutineContext

interface Server {
    val eventBus: EventBus
    val eventHandlerPositions: EventHandlerPositionManager

    /**
     * [CoroutineContext] confined to the server thread.
     *
     * Is cancelled when the server is shutting down.
     */
    val coroutineContext: CoroutineContext

    /**
     * All sessions connected to the server.
     */
    val sessions: Collection<Session>

    /**
     * All players connected to the server.
     */
    val players: Collection<Player>

    val pluginManager: PluginManager
    val serverDirectory: File

    val dimensionRegistry: Registry<Dimension>
    val recipeRegistry: Registry<Recipe>
    val commandRegistry: Registry<Command>
    val biomeRegistry: BiomeRegistry

    val loggingOutputProvider: LoggingOutputProvider
    val scheduler: Scheduler

    val developmentMode: Boolean
    val minimumLogLevel: Logger.Level

    /**
     * Initiates shutting down the server.
     */
    fun shutdown()

    companion object {
        const val TICKS_PER_SECOND = 20
    }
}
