package space.blokk.tag

import space.blokk.entity.EntityType
import space.blokk.item.ItemType
import space.blokk.world.NUMERIC_FLUID_IDS_BY_ID
import space.blokk.world.block.Material

// TODO: Allow plugins to create custom tags and actually use tag values server-side
class Tag(val name: String, val type: Type, val rawValues: List<String>) {
    val values: List<String> by lazy {
        val tags = TagRegistry.tagsByNameByType.getValue(type)

        rawValues.flatMap {
            if (it.startsWith("#")) tags.getValue(it.removePrefix("#")).values
            else listOf(it)
        }
    }

    val numericIDs: List<Int> by lazy {
        when (type) {
            Type.BLOCKS -> values.map { Material.byID.getValue(it).codec.id }
            Type.ENTITY_TYPES -> values.map { EntityType.byID.getValue(it).numericID }
            Type.FLUIDS -> values.map { NUMERIC_FLUID_IDS_BY_ID.getValue(it) }
            Type.ITEMS -> values.map { ItemType.byID.getValue(it).numericID }
        }
    }

    enum class Type {
        BLOCKS,
        ENTITY_TYPES,
        FLUIDS,
        ITEMS
    }
}
