package space.blokk.net.packet.play

import io.netty.buffer.ByteBuf
import io.netty.buffer.ByteBufOutputStream
import io.netty.buffer.Unpooled
import space.blokk.Blokk
import space.blokk.nbt.NBT
import space.blokk.nbt.NBTCompound
import space.blokk.net.MinecraftProtocolDataTypes.writeNBT
import space.blokk.net.MinecraftProtocolDataTypes.writeVarInt
import space.blokk.net.packet.OutgoingPacketCodec
import space.blokk.util.generateHeightmap
import space.blokk.util.setBit
import space.blokk.util.toCompactLongArray
import space.blokk.world.block.Air
import space.blokk.world.block.CaveAir
import space.blokk.world.block.Material
import space.blokk.world.block.material
import java.io.DataOutputStream
import java.io.File
import java.util.zip.Deflater
import kotlin.math.ceil
import kotlin.math.log

object ChunkDataPacketCodec : OutgoingPacketCodec<ChunkDataPacket>(0x20, ChunkDataPacket::class) {
    private val airBlocks: Set<Material<*>> = setOf(Air, CaveAir)
    private val nonSolidBlocks = Material.all.filter { it.collisionShape.isEmpty() }

    private val numberOfBitsPerBlock = ceil(log(Material.all.last().codec.lastStateID.toDouble(), 2.0)).toInt()

    override fun ChunkDataPacket.encode(dst: ByteBuf) {
        dst.writeInt(key.x)
        dst.writeInt(key.z)
        dst.writeBoolean(true) // Full Chunk

        // Included sections bitmask
        val mask = data.sections.foldIndexed(0) { i, acc, current -> acc.setBit(i, current != null) }
        dst.writeVarInt(mask)

        // Heightmaps
        val heightmaps = NBT()
        heightmaps.set(
            "MOTION_BLOCKING",
            (data.solidBlocksHeightmap ?: generateHeightmap(
                data.sections,
                nonSolidBlocks
            )).toCompactLongArray(9).toList()
        )

        heightmaps.set(
            "WORLD_SURFACE",
            (data.nonAirBlocksHeightmap ?: generateHeightmap(
                data.sections,
                airBlocks
            )).toCompactLongArray(9)
        )

        dst.writeNBT(heightmaps)

        // Biomes
        dst.writeVarInt(data.biomes.size)
        data.biomes.forEach {
            dst.writeVarInt(it.numericID ?: throw IllegalStateException("A biome in the chunk was not registered"))
        }

        // TODO: Use palettes

        // Blocks
        val dataBuf = Unpooled.buffer() // TODO: Set an initial capacity
        for (section in data.sections.filterNotNull()) {
            val ids = section.map { it.material().codec.getStateID(it) }.toIntArray()
            dataBuf.writeShort(ids.count { !(it == Air.codec.id || it == CaveAir.codec.id) })
            dataBuf.writeByte(numberOfBitsPerBlock)

            val array = ids.toCompactLongArray(numberOfBitsPerBlock)
            dataBuf.writeVarInt(array.size)
            array.forEach { dataBuf.writeLong(it) }
        }

        dst.writeVarInt(dataBuf.readableBytes())
        dst.writeBytes(dataBuf)
        dataBuf.release()

        // Block entities
        // TODO: Send Block entities
        dst.writeVarInt(0)
    }
}
