package space.blokk.net.packet.play

import io.netty.buffer.ByteBuf
import space.blokk.net.MinecraftProtocolDataTypes.readString
import space.blokk.net.MinecraftProtocolDataTypes.readVarInt
import space.blokk.net.PacketDecodingException
import space.blokk.net.packet.IncomingPacketCodec
import space.blokk.player.ChatMode
import space.blokk.player.Hand
import space.blokk.player.SkinPartsConfiguration
import space.blokk.util.checkBit

object ClientSettingsPacketCodec : IncomingPacketCodec<ClientSettingsPacket>(0x05, ClientSettingsPacket::class) {
    override fun decode(msg: ByteBuf): ClientSettingsPacket {
        val locale = msg.readString()
        val viewDistance = msg.readByte()

        val chatMode = when (msg.readVarInt()) {
            0 -> ChatMode.ENABLED
            1 -> ChatMode.COMMANDS_ONLY
            2 -> ChatMode.HIDDEN
            else -> throw PacketDecodingException()
        }

        val chatColorsEnabled = msg.readBoolean()

        val skinFlags = msg.readByte()
        val displayedSkinParts = SkinPartsConfiguration(
            cape = skinFlags.checkBit(0),
            jacket = skinFlags.checkBit(1),
            leftSleeve = skinFlags.checkBit(2),
            rightSleeve = skinFlags.checkBit(3),
            leftPantsLeg = skinFlags.checkBit(4),
            rightPantsLeg = skinFlags.checkBit(5),
            hat = skinFlags.checkBit(6)
        )

        val mainHand = when (msg.readVarInt()) {
            0 -> Hand.LEFT
            1 -> Hand.RIGHT
            else -> throw PacketDecodingException()
        }

        return ClientSettingsPacket(
            locale,
            viewDistance,
            chatMode,
            chatColorsEnabled,
            displayedSkinParts,
            mainHand
        )
    }
}
