package space.blokk.net.packet.play

import space.blokk.chat.TextComponent
import space.blokk.net.packet.OutgoingPacket
import space.blokk.player.GameMode
import java.util.*

/**
 * Informs the client about other players.
 */
data class PlayerInfoPacket(val action: Action<*>) : OutgoingPacket() {
    sealed class Action<T> {
        abstract val entries: Map<UUID, T>

        data class AddPlayer(override val entries: Map<UUID, Data>) : Action<AddPlayer.Data>() {
            data class Data(
                val name: String,
                /**
                 * If this does not match the real game mode of a player, weird things can happen.
                 *
                 * Use with caution.
                 */
                val gameMode: GameMode,
                /**
                 * Latency displayed in the player list in milliseconds.
                 *
                 * See [LATENCY] for constants.
                 */
                val latency: Int,
                /**
                 * Only visible in the player list.
                 */
                val displayName: TextComponent?,
                /**
                 * Primarily used for the player skin (key: textures).
                 */
                val properties: Map<String, Property>
            ) {
                data class Property(
                    val value: String,
                    val signature: String? = null
                )
            }
        }

        /**
         * See [AddPlayer.Data.gameMode].
         */
        data class UpdateGameMode(override val entries: Map<UUID, GameMode>) : Action<GameMode>()

        /**
         * See [AddPlayer.Data.latency].
         */
        data class UpdateLatency(override val entries: Map<UUID, Int>) : Action<Int>()

        /**
         * See [AddPlayer.Data.displayName].
         */
        data class UpdateDisplayName(override val entries: Map<UUID, TextComponent?>) : Action<TextComponent?>()

        data class RemovePlayer(override val entries: Map<UUID, Unit>): Action<Unit>() {
            constructor(entries: List<UUID>): this(entries.map { it to Unit }.toMap())
        }
    }

    /**
     * Latency constants for use in the [AddPlayer][Action.AddPlayer] and [UpdateLatency][Action.UpdateLatency] actions.
     */
    object LATENCY {
        const val NO_CONNECTION = -1
        const val FIVE_BARS = 0
        const val FOUR_BARS = 150
        const val THREE_BARS = 300
        const val TWO_BARS = 600
        const val ONE_BAR = 1000
    }
}
