plugins {
    kotlin("jvm")
    kotlin("kapt")
    id("com.github.johnrengelman.shadow") version "6.1.0"
}

group = rootProject.group
version = rootProject.version

repositories {
    mavenCentral()
    jcenter()
    maven("https://jitpack.io")
}

val coroutinesVersion = properties["version.kotlinx-coroutines"].toString()
val slf4jVersion = properties["version.slf4j"].toString()
val nettyVersion = properties["version.netty"].toString()
val moshiVersion = properties["version.moshi"].toString()
val junitVersion = properties["version.junit"].toString()
val striktVersion = properties["version.strikt"].toString()

dependencies {
    // Kotlin
    implementation("org.jetbrains.kotlinx:kotlinx-coroutines-core:${coroutinesVersion}")
    implementation("org.jetbrains.kotlinx:kotlinx-coroutines-jdk8:${coroutinesVersion}")

    // Blokk
    implementation(project(":blokk-api"))
    implementation(project(":blokk-packets"))
    implementation(project(":blokk-packet-codecs"))
    implementation(project(":blokk-nbt"))

    // Logging
    implementation("org.slf4j:slf4j-api:${slf4jVersion}")
    implementation("ch.qos.logback:logback-classic:1.2.3")
    implementation("de.moritzruth:khalk:1.0.0-fix")

    // Netty
    implementation("io.netty:netty-handler:${nettyVersion}")
    implementation("io.netty:netty-buffer:${nettyVersion}")
    implementation("io.netty:netty-codec:${nettyVersion}")
    implementation("io.netty:netty-transport:${nettyVersion}")
    implementation("io.netty:netty-transport-native-epoll:${nettyVersion}")
    implementation("io.netty:netty-transport-native-kqueue:${nettyVersion}")

    // Other
    implementation("com.squareup.okhttp3:okhttp:4.9.0")
    implementation("com.sksamuel.hoplite:hoplite-core:1.3.9")
    implementation("com.sksamuel.hoplite:hoplite-yaml:1.3.9")
    kapt("com.squareup.moshi:moshi-kotlin-codegen:${moshiVersion}")

    // Testing
    testImplementation("io.strikt:strikt-core:${striktVersion}")
    testImplementation("org.junit.jupiter:junit-jupiter-api:${junitVersion}")
    testRuntimeOnly("org.junit.jupiter:junit-jupiter-engine:${junitVersion}")
}

tasks {
    compileKotlin {
        kotlinOptions.freeCompilerArgs = listOf(
            "-Xopt-in=kotlin.RequiresOptIn",
            "-progressive"
        )
    }

    test {
        useJUnitPlatform()
    }

    shadowJar {
        archiveClassifier.set(null as String?)

        @Suppress("UnstableApiUsage")
        manifest {
            this.attributes(
                "Main-Class" to "space.blokk.BlokkServer",
                "Implementation-Version" to project.version
            )
        }
    }
}
