package space.blokk.net

import io.netty.bootstrap.ServerBootstrap
import io.netty.channel.ChannelOption
import io.netty.channel.epoll.Epoll
import io.netty.channel.epoll.EpollEventLoopGroup
import io.netty.channel.epoll.EpollServerSocketChannel
import io.netty.channel.kqueue.KQueue
import io.netty.channel.kqueue.KQueueEventLoopGroup
import io.netty.channel.kqueue.KQueueServerSocketChannel
import io.netty.channel.nio.NioEventLoopGroup
import io.netty.channel.socket.nio.NioServerSocketChannel
import space.blokk.BlokkServer
import space.blokk.logging.Logger
import java.util.concurrent.CopyOnWriteArraySet

class BlokkSocketServer(private val server: BlokkServer) {
    private val bossGroup = createEventLoopGroup()
    private val workerGroup = createEventLoopGroup()
    private val bootstrap: ServerBootstrap = ServerBootstrap()
        .group(bossGroup, workerGroup)
        .channel(
            when {
                EPOLL_AVAILABLE -> EpollServerSocketChannel::class.java
                KQUEUE_AVAILABLE -> KQueueServerSocketChannel::class.java
                else -> NioServerSocketChannel::class.java
            }
        )
        .childOption(ChannelOption.SO_KEEPALIVE, true)
        .childOption(ChannelOption.TCP_NODELAY, true)
        .childHandler(BlokkChannelInitializer(server))

    internal val sessions: MutableSet<BlokkSession> = CopyOnWriteArraySet()

    fun bind() {
        bootstrap.bind(server.config.host, server.config.port)
    }

    companion object {
        private val EPOLL_AVAILABLE = Epoll.isAvailable()
        private val KQUEUE_AVAILABLE = KQueue.isAvailable()
        private fun createEventLoopGroup() = when {
            EPOLL_AVAILABLE -> EpollEventLoopGroup()
            KQUEUE_AVAILABLE -> KQueueEventLoopGroup()
            else -> NioEventLoopGroup()
        }
    }
}
