package space.blokk.net

import io.netty.buffer.ByteBuf
import io.netty.channel.ChannelHandlerContext
import io.netty.handler.codec.ByteToMessageCodec
import space.blokk.BlokkServer
import space.blokk.net.MinecraftProtocolDataTypes.readVarInt
import space.blokk.net.MinecraftProtocolDataTypes.varIntReadable
import space.blokk.net.MinecraftProtocolDataTypes.writeVarInt

class FramingCodec(private val server: BlokkServer) : ByteToMessageCodec<ByteBuf>() {
    private var currentLength: Int? = null

    override fun channelActive(ctx: ChannelHandlerContext) {
        val session = BlokkSession(ctx.channel(), server)

        ctx.channel().pipeline()
            .addLast("packets", PacketMessageDuplexHandler(session))

        ctx.fireChannelActive()
    }

    override fun encode(ctx: ChannelHandlerContext, msg: ByteBuf, out: ByteBuf) {
        out.writeVarInt(msg.readableBytes())
        msg.readerIndex(0)
        out.writeBytes(msg)
    }

    override fun decode(ctx: ChannelHandlerContext, msg: ByteBuf, out: MutableList<Any>) {
        var length = currentLength

        if (length == null) {
            if (msg.varIntReadable()) {
                length = msg.readVarInt()
                currentLength = length
            } else return
        }

        if (msg.readableBytes() >= length) {
            out.add(msg.readBytes(length))
            currentLength = null
        }
    }
}
