package space.blokk.net

import io.netty.buffer.ByteBuf
import io.netty.channel.ChannelDuplexHandler
import io.netty.channel.ChannelHandlerContext
import io.netty.channel.ChannelPromise
import io.netty.handler.codec.MessageToMessageCodec
import space.blokk.net.MinecraftProtocolDataTypes.readVarInt
import space.blokk.net.MinecraftProtocolDataTypes.writeVarInt
import space.blokk.net.packet.OutgoingPacket
import space.blokk.net.packet.OutgoingPacketCodec
import java.io.IOException

class PacketMessageDuplexHandler(private val session: BlokkSession) : ChannelDuplexHandler() {
    override fun channelActive(ctx: ChannelHandlerContext) {
        session.onConnect()
    }

    override fun channelInactive(ctx: ChannelHandlerContext) {
        session.onDisconnect(false)
    }

    override fun write(ctx: ChannelHandlerContext, msg: Any, promise: ChannelPromise) {
        val packet = msg as OutgoingPacket

        @Suppress("UNCHECKED_CAST")
        val codec =
            session.currentProtocol!!.getCodecByType(packet::class) as OutgoingPacketCodec<OutgoingPacket>

        val dst = ctx.alloc().buffer()
        dst.writeVarInt(codec.id)
        codec.encode(packet, dst)
        ctx.write(dst, promise)
    }

    override fun channelRead(ctx: ChannelHandlerContext, msg: Any) {
        // TODO: Check if the ByteBuf needs to be released
        session.onPacketDataReceived(msg as ByteBuf)
    }

    override fun exceptionCaught(ctx: ChannelHandlerContext, cause: Throwable) {
        session.failBecauseOfClient("${cause::class.java.name}: ${cause.message}")
    }
}
