package space.blokk.player

import space.blokk.chat.TextComponent
import space.blokk.net.Session
import space.blokk.net.packet.play.ChunkDataPacket
import space.blokk.net.packet.play.ChunkLightDataPacket
import space.blokk.world.Chunk
import space.blokk.world.LocationWithRotation
import space.blokk.world.VoxelLocation
import space.blokk.world.World
import java.util.*
import kotlin.math.abs

class BlokkPlayer(
    override val session: Session,
    override val name: String,
    override val uuid: UUID,
    override var gameMode: GameMode,
    override var settings: Player.Settings,
    override val world: World,
    override var location: LocationWithRotation,
    override var reducedDebugInfo: Boolean,
    override var fieldOfView: Float,
    override var canFly: Boolean,
    override var flying: Boolean,
    override var flyingSpeed: Float,
    override var invulnerable: Boolean,
    override var compassTarget: VoxelLocation,
    selectedHotbarSlot: Int
) : Player {
    override var selectedHotbarSlot = 0
        set(value) {
            if (value in 0..8) field = value
            else throw IllegalArgumentException("selectedHotbarSlot must be in 0..8")
        }

    init {
        this.selectedHotbarSlot = selectedHotbarSlot
    }

    override var playerListName: TextComponent? = null
    override var currentlyViewedChunks: List<Chunk> = emptyList()

    init {
        updateCurrentlyViewedChunks()
    }

    /**
     * Sets [currentlyViewedChunks] to all chunks in the view distance.
     */
    @OptIn(ExperimentalStdlibApi::class)
    fun updateCurrentlyViewedChunks() {
        val (centerX, centerZ) = Chunk.Key.from(location.toVoxelLocation())

        val edgeLength = settings.viewDistance + 1

        currentlyViewedChunks = buildList(edgeLength * edgeLength) {
            for (x in (centerX - settings.viewDistance)..(centerX + settings.viewDistance)) {
                for (z in (centerZ - settings.viewDistance)..(centerZ + settings.viewDistance)) {
                    add(world.getChunk(Chunk.Key(x, z)))
                }
            }
        }
    }

    suspend fun sendChunksAndLight() {
        val chunks = currentlyViewedChunks.sortedBy { abs(it.key.x) + abs(it.key.z) }
        chunks.forEach { session.send(ChunkLightDataPacket(it.key, it.getLightData(this))) }
        chunks.forEach { session.send(ChunkDataPacket(it.key, it.getData(this))) }
    }
}
