package space.blokk.mdsp.generator

import com.google.common.base.CaseFormat
import com.jsoniter.JsonIterator
import com.squareup.kotlinpoet.*
import com.squareup.kotlinpoet.ParameterizedTypeName.Companion.parameterizedBy
import space.blokk.mdsp.JsonAny
import java.io.File

class EntitiesGenerator(
    private val workingDir: File,
    private val outputDir: File,
    private val sourcesDir: File
) {
    fun generate() {
        val entitiesJson = workingDir.resolve("entities.json").readText()
        val entities = JsonIterator.deserialize(entitiesJson).asList()

        generateEntityStubs(entities)
        generateEntitiesList(entities)
    }

    private val typeLines = listOf("%L", "%S", "%Lf", "%Lf")

    private fun generateEntityStubs(entities: List<JsonAny>) {
        for (entity in entities) {
            val name = CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, entity.get("name").toString())!!

            val filePathRelativeToSourceRoot = "./${BLOCK_PACKAGE.replace(".", "/")}/$name.kt"
            if (sourcesDir.resolve(filePathRelativeToSourceRoot).exists()) continue

            val typeArgs = arrayOf(
                entity.get("id").toInt(),
                "minecraft:" + entity.get("name").toString(),
                entity.get("width").toFloat(),
                entity.get("height").toFloat()
            )

            val type = TypeSpec.classBuilder(name)
                .superclass(ENTITY_TYPE)
                .addType(
                    TypeSpec.companionObjectBuilder()
                        .addSuperinterface(
                            ENTITY_TYPE_TYPE.parameterizedBy(ClassName(ENTITY_PACKAGE, name)),
                            CodeBlock.of("type(\n${typeLines.joinToString(",\n") { "    $it" }}\n)", *typeArgs)
                        )
                        .build()
                )
                .build()

            FileSpec.builder(ENTITY_PACKAGE, name)
                .addType(type)
                .build()
                .writeTo(outputDir)
        }
    }

    private fun generateEntitiesList(entities: List<JsonAny>) {
        val names = entities
            .map { it.get("name").toString() }
            .map { CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, it) }

        val property = PropertySpec.builder(
            "GENERATED_ENTITIES",
            List::class.asTypeName().parameterizedBy(ENTITY_TYPE_TYPE.parameterizedBy(STAR))
        )
            .initializer("listOf(\n${names.joinToString(",\n")}\n)")
            .build()

        FileSpec.builder(ENTITY_PACKAGE, "Entities")
            .addProperty(property)
            .build()
            .writeTo(outputDir)
    }
}
