package space.blokk.testplugin

import space.blokk.Blokk
import space.blokk.chat.TextComponent
import space.blokk.net.ServerListInfo
import space.blokk.net.event.ServerListInfoRequestEvent
import space.blokk.net.event.SessionAfterLoginEvent
import space.blokk.player.GameMode
import space.blokk.plugin.Plugin
import space.blokk.testplugin.anvil.AnvilWorld
import space.blokk.world.Dimension
import space.blokk.world.VoxelLocation
import space.blokk.world.WorldAndLocationWithRotation
import space.blokk.world.block.CraftingTable
import space.blokk.world.block.GreenWool
import space.blokk.world.block.RedWool

class TestPlugin: Plugin("Test", "1.0.0") {
    override fun onEnable() {
        val dimension = Dimension(
            "test:test",
            true,
            0.0f,
            true
        )

        Blokk.dimensionRegistry.register(dimension)

        val world = AnvilWorld(dimension, true)
        world.getVoxelsInCube(VoxelLocation.of(16, 0, 16), VoxelLocation.of(-16, 0, -16)).forEach {
            it.block = if (it.location.x % 16 == 0 || it.location.z % 16 == 0) GreenWool() else RedWool()
        }

        world.getVoxel(VoxelLocation.of(-1, 2, -1)).block = CraftingTable()

        Blokk.eventBus.on<ServerListInfoRequestEvent> { event ->
            event.response = ServerListInfo("1.16.4", 754, TextComponent of "Test", 10, 0, emptyList())
        }

        Blokk.eventBus.on<SessionAfterLoginEvent> { event ->
            event.initialWorldAndLocation = WorldAndLocationWithRotation(
                world,
                VoxelLocation.of(0, 2, 0).atTopCenter().withRotation(0f, 0f)
            )
        }
    }
}
