package space.blokk.testplugin.anvil

import com.google.common.cache.LoadingCache
import space.blokk.player.Player
import space.blokk.util.createWeakValuesLoadingCache
import space.blokk.world.*
import space.blokk.world.block.Air
import space.blokk.world.block.Block

class AnvilChunk(world: AnvilWorld, key: Key) : Chunk(world, key) {
    // TODO: Implement light

    override fun getData(player: Player?): ChunkData {
        return ChunkData(
            sections.map { section -> if (section.blocks.all { it == Air }) null else section.blocks }.toTypedArray(),
            Array(ChunkData.BIOME_AREAS_IN_CHUNK) { Biome.PLAINS }
        )
    }

    override fun getLightData(player: Player?): ChunkLightData {
        return ChunkLightData(
            sections.map { it.skyLightValues }.toTypedArray(),
            sections.map { it.blockLightValues }.toTypedArray()
        )
    }

    val sections: Array<AnvilChunkSection> = Array(16) { AnvilChunkSection() }

    // Keys are relative to the world
    private val voxels: LoadingCache<VoxelLocation, Voxel> =
        createWeakValuesLoadingCache { location -> AnvilVoxel(location, world) }

    override fun getVoxelRelativeToChunk(location: VoxelLocation): Voxel {
        return voxels.get(key.translateChunkToWorld(location))
    }

    override fun getVoxel(location: VoxelLocation): Voxel {
        return voxels.get(location)
    }
}
